import configparser
import dataclasses
from enum import Enum
from typing import Any, Dict, List, Optional

import pydantic
from pydantic import BaseModel


class FieldDataType(BaseModel):
    meta: Dict[str, Any]
    type_: str = pydantic.Field(..., alias="type")


class FieldDefinition(BaseModel):
    name: str
    description: Optional[str] = None
    type_: str = pydantic.Field(..., alias="type")
    primary: bool
    optional: bool
    data_type: FieldDataType


class DataProductCreate(BaseModel):
    engine: str
    fields: List[FieldDefinition]


class RegisterCore(BaseModel):
    partition: str
    name: str


class RemoveCore(BaseModel):
    urn: str


class Auth(BaseModel):
    access_token: str = ""
    expires_in: Optional[int] = None
    refresh_token: str = ""
    refresh_expires_in: Optional[int] = None


class OptionalProfile(BaseModel):
    gateway_api_url: str = ""
    registry_api_url: str = ""
    iam_api_url: str = ""
    storage_api_url: str = ""
    user: str = ""
    access_token: str = ""
    refresh_token: str = ""
    ignore_tls: bool = False


class Profile(BaseModel):
    gateway_api_url: str
    registry_api_url: str
    iam_api_url: str
    storage_api_url: str
    user: str
    access_token: str
    refresh_token: str
    ignore_tls: bool


class EffectEnum(Enum):
    allow: str = "allow"
    deny: str = "deny"


class Statement(BaseModel):
    sid: str
    principal: List[str]
    action: List[str]
    resource: List[str]
    condition: Optional[List[str]] = None
    effect: EffectEnum = EffectEnum.allow

    class Config:
        use_enum_values = True


class Statements(BaseModel):
    statements: List[Statement]


class Policy(BaseModel):
    version: str = "2022-10-01"
    statements: List[Statement]


class UserPolicy(BaseModel):
    user: str
    policy: Policy


@dataclasses.dataclass
class Common:
    gateway_api_url: str
    registry_api_url: str
    iam_api_url: str
    storage_api_url: str
    profile_name: str
    config: configparser.ConfigParser
    profile: Optional[Profile]

    def get_gateway_api_url(self):
        url = None
        if self.gateway_api_url:
            url = self.gateway_api_url
        elif self.profile and self.profile.gateway_api_url:
            url = self.profile.gateway_api_url
        return url  # noqa: RET504

    def get_registry_api_url(self):
        url = None
        if self.registry_api_url:
            url = self.registry_api_url
        if self.profile and self.profile.registry_api_url:
            url = self.profile.registry_api_url
        return url  # noqa: RET504

    def get_iam_api_url(self):
        url = None
        if self.iam_api_url:
            url = self.iam_api_url
        if self.profile and self.profile.iam_api_url:
            url = self.profile.iam_api_url
        return url  # noqa: RET504

    def get_storage_api_url(self):
        url = None
        if self.storage_api_url:
            url = self.storage_api_url
        if self.profile and self.profile.storage_api_url:
            url = self.profile.storage_api_url
        return url  # noqa: RET504



import configparser
import json
import pathlib
import typing

import httpx
import pydantic
import typer
from pygments import formatters, highlight, lexers

from neosctl import constant, schema


def dumps_formatted_json(payload: typing.Dict):
    return json.dumps(payload, indent=2, sort_keys=True)


def prettify_json(payload: typing.Dict) -> str:
    return highlight(dumps_formatted_json(payload), lexers.JsonLexer(), formatters.TerminalFormatter())


def is_success_response(response: httpx.Response):
    if constant.SUCCESS_CODE <= response.status_code < constant.REDIRECT_CODE:
        return True
    return False


def send_output(msg: str, exit_code: int = 0):
    typer.echo(msg)

    raise typer.Exit(exit_code)


def process_response(response: httpx.Response, render_callable=prettify_json):
    exit_code = 0
    data = response.json()
    if response.status_code >= constant.BAD_REQUEST_CODE:
        exit_code = 1
        message = prettify_json(data)
    else:
        message = render_callable(data)

    send_output(
        msg=message,
        exit_code=exit_code,
    )


def read_config_dotfile() -> configparser.ConfigParser:
    c = configparser.ConfigParser()
    c.read(constant.PROFILE_FILEPATH)
    return c


def get_schema_profile(profile_name, allow_missing: bool = False, **kwargs):
    if allow_missing:
        return schema.OptionalProfile(**kwargs)

    try:
        return schema.Profile(**kwargs)
    except pydantic.ValidationError as e:
        required_fields = [err["loc"][0] for err in e.errors() if err["msg"] == "field required"]
        send_output(
            msg=("Profile dotfile doesn't include fields:\n  {}\nUse neosctl -p {} profile init").format(
                "\n  ".join(required_fields),
                profile_name,
            ),
            exit_code=1,
        )


def get_user_profile_section(c: configparser.ConfigParser, profile_name: str, allow_missing: bool = False):
    try:
        return c[profile_name]
    except KeyError:
        if not allow_missing:
            send_output(
                msg=f"Profile {profile_name} not found.",
                exit_code=1,
            )


def get_user_profile(
    c: configparser.ConfigParser,
    profile_name: str,
    allow_missing: bool = False,
) -> typing.Optional[schema.Profile]:
    profile_config = get_user_profile_section(c, profile_name, allow_missing=allow_missing)
    if profile_config:
        return get_schema_profile(profile_name, allow_missing=allow_missing, **profile_config)
    return None


def bearer(ctx: typer.Context) -> typing.Optional[typing.Dict]:
    if not (ctx.obj.profile and ctx.obj.profile.access_token != ""):  # nosec: B105
        return None

    return {"Authorization": f"Bearer {ctx.obj.profile.access_token}"}


def check_profile_exists(ctx: typer.Context):
    if not ctx.obj.profile:
        send_output(
            msg=f"Profile not found! Run neosctl -p {ctx.obj.profile_name} profile init",
            exit_code=1,
        )

    return True


def upsert_config(
    ctx: typer.Context,
    profile: schema.Profile,
) -> configparser.ConfigParser:
    ctx.obj.config[ctx.obj.profile_name] = profile.dict()

    with constant.PROFILE_FILEPATH.open("w") as profile_file:
        ctx.obj.config.write(profile_file)

    return ctx.obj.config


def remove_config(
    ctx: typer.Context,
) -> configparser.ConfigParser:
    if not ctx.obj.config.remove_section(ctx.obj.profile_name):
        send_output(
            msg=f"Can not remove {ctx.obj.profile_name} profile, profile not found.",
            exit_code=1,
        )

    with constant.PROFILE_FILEPATH.open("w") as profile_file:
        ctx.obj.config.write(profile_file)

    return ctx.obj.config


def get_file_location(filepath: str) -> pathlib.Path:
    fp = pathlib.Path(filepath)
    if not fp.exists():
        send_output(
            msg=f"Can not find file: {fp}",
            exit_code=1,
        )
    return fp


def load_json_file(fp: pathlib.Path, content_type: str) -> typing.Union[typing.Dict, typing.List]:
    with fp.open() as f:
        try:
            data = json.load(f)
        except json.decoder.JSONDecodeError:
            send_output(
                msg=f"Invalid {content_type} file, must be json format.",
                exit_code=1,
            )
    return data


def _request(ctx: typer.Context, method: str, url: str, **kwargs):
    return httpx.request(method=method, url=url, headers=bearer(ctx), verify=not ctx.obj.profile.ignore_tls, **kwargs)


def get(ctx: typer.Context, url: str, **kwargs):
    return _request(ctx, "GET", url, **kwargs)


def post(ctx: typer.Context, url: str, **kwargs):
    return _request(ctx, "POST", url, **kwargs)


def put(ctx: typer.Context, url: str, **kwargs):
    return _request(ctx, "PUT", url, **kwargs)


def patch(ctx: typer.Context, url: str, **kwargs):
    return _request(ctx, "PATCH", url, **kwargs)


def delete(ctx: typer.Context, url: str, **kwargs):
    return _request(ctx, "DELETE", url, **kwargs)



import typing

import httpx
import typer

from neosctl import util
from neosctl.auth import ensure_login
from neosctl.util import process_response

app = typer.Typer()


def storage_url(ctx: typer.Context, postfix: str) -> str:
    return "{}/{}".format(ctx.obj.storage_api_url.rstrip("/"), postfix)


def _load_statement(statement: typing.Optional[str], statement_filepath: typing.Optional[str]):
    if statement is None and statement_filepath is None:
        util.send_output(
            msg="At least one of --statement/--statement-filepath is required.",
            exit_code=1,
        )

    if statement_filepath:
        fp = util.get_file_location(statement_filepath)

        return fp.read_text()

    return statement


def _load_params(params_filepath: typing.Optional[str]):
    if params_filepath:
        fp = util.get_file_location(params_filepath)

        return util.load_json_file(fp, "params")

    return None


def _handle(
    ctx: typer.Context,
    postfix: str,
    statement: typing.Optional[str] = None,
    statement_filepath: typing.Optional[str] = None,
    params_filepath: typing.Optional[str] = None,
):
    @ensure_login
    def _request(ctx: typer.Context, params: typing.List[typing.Any]) -> httpx.Response:
        return util.post(
            ctx,
            url=storage_url(ctx, postfix),
            json={
                "statement": statement,
                "params": params,
            },
        )

    statement = _load_statement(statement, statement_filepath)
    params = _load_params(params_filepath)

    r = _request(ctx, params)
    process_response(r)


@app.command()
def execute(
    ctx: typer.Context,
    statement: typing.Optional[str] = typer.Option(None, "--statement", "-s", help="pSQL statement"),
    statement_filepath: typing.Optional[str] = typer.Option(
        None,
        "--statement-filepath",
        "-sf",
        help="Filepath for statement sql file.",
    ),
    params_filepath: typing.Optional[str] = typer.Option(
        None,
        "--params-filepath",
        "-pf",
        help="Filepath for statement params json file.",
    ),
):
    """Execute a statement."""
    _handle(ctx, "execute", statement, statement_filepath, params_filepath)


@app.command()
def executemany(
    ctx: typer.Context,
    statement: typing.Optional[str] = typer.Option(None, "--statement", "-s", help="pSQL statement"),
    statement_filepath: typing.Optional[str] = typer.Option(
        None,
        "--statement-filepath",
        "-sf",
        help="Filepath for statement sql file.",
    ),
    params_filepath: typing.Optional[str] = typer.Option(
        None,
        "--params-filepath",
        "-pf",
        help="Filepath for statement params json file.",
    ),
):
    """Execute a statement with multiple input params."""
    _handle(ctx, "executemany", statement, statement_filepath, params_filepath)


@app.command()
def fetch(
    ctx: typer.Context,
    statement: typing.Optional[str] = typer.Option(None, "--statement", "-s", help="pSQL statement"),
    statement_filepath: typing.Optional[str] = typer.Option(
        None,
        "--statement-filepath",
        "-sf",
        help="Filepath for statement sql file.",
    ),
    params_filepath: typing.Optional[str] = typer.Option(
        None,
        "--params-filepath",
        "-pf",
        help="Filepath for statement params json file.",
    ),
):
    """Fetch results of a statement."""
    _handle(ctx, "fetch", statement, statement_filepath, params_filepath)


@app.command()
def fetchrow(
    ctx: typer.Context,
    statement: typing.Optional[str] = typer.Option(None, "--statement", "-s", help="pSQL statement"),
    statement_filepath: typing.Optional[str] = typer.Option(
        None,
        "--statement-filepath",
        "-sf",
        help="Filepath for statement sql file.",
    ),
    params_filepath: typing.Optional[str] = typer.Option(
        None,
        "--params-filepath",
        "-pf",
        help="Filepath for statement params json file.",
    ),
):
    """Fetch first result of a statement."""
    _handle(ctx, "fetchrow", statement, statement_filepath, params_filepath)



import typing

import httpx
import typer

from neosctl import schema, util
from neosctl.auth import ensure_login
from neosctl.util import process_response

app = typer.Typer()


def iam_url(iam_api_url: str, postfix: str = "") -> str:
    return "{}/{}".format(iam_api_url.rstrip("/"), postfix)


@app.command(name="list")
def list_policies(
    ctx: typer.Context,
    page: int = typer.Option(1, help="Page number."),
    page_size: int = typer.Option(10, help="Page size number."),
    resource: typing.Optional[str] = typer.Option(None, help="Resource nrn."),
):
    """List existing policies."""

    @ensure_login
    def _request(ctx: typer.Context):
        params = {"page": page, "page_size": page_size}
        if resource:
            params["resource"] = resource

        return util.get(
            ctx,
            iam_url(ctx.obj.get_iam_api_url(), "policy/users"),
            params=params,
        )

    r = _request(ctx)
    process_response(r)


@app.command(name="create")
def create_from_json(
    ctx: typer.Context,
    filepath: str = typer.Argument(..., help="Filepath of the user policy json payload"),
):
    """Create an IAM policy."""

    @ensure_login
    def _request(ctx: typer.Context, user_policy: schema.UserPolicy) -> httpx.Response:
        return util.post(
            ctx,
            "{iam_url}".format(iam_url=iam_url(ctx.obj.get_iam_api_url(), "policy/user")),
            json=user_policy.dict(),
        )

    fp = util.get_file_location(filepath)
    user_policy_payload = util.load_json_file(fp, "schema")

    user_policy = schema.UserPolicy(**user_policy_payload)

    r = _request(ctx, user_policy)
    process_response(r)


@app.command(name="update")
def update_from_json(
    ctx: typer.Context,
    principal: str = typer.Argument(..., help="Principal uuid"),
    filepath: str = typer.Argument(..., help="Filepath of the user policy json payload"),
):
    """Update an existing IAM policy."""

    @ensure_login
    def _request(ctx: typer.Context, user_policy: schema.UserPolicy) -> httpx.Response:
        return util.put(
            ctx,
            "{iam_url}".format(iam_url=iam_url(ctx.obj.get_iam_api_url(), "policy/user")),
            params={"user_nrn": principal},
            json=user_policy.dict(),
        )

    fp = util.get_file_location(filepath)
    user_policy_payload = util.load_json_file(fp, "schema")

    user_policy = schema.UserPolicy(**user_policy_payload)

    r = _request(ctx, user_policy)
    process_response(r)


@app.command()
def delete(ctx: typer.Context, user_nrn: str):
    """Delete an existing IAM policy."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            "{iam_url}".format(iam_url=iam_url(ctx.obj.get_iam_api_url(), "policy/user")),
            params={"user_nrn": user_nrn},
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def get(ctx: typer.Context, user_nrn: str):
    """Get an existing IAM policy."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            "{iam_url}".format(iam_url=iam_url(ctx.obj.get_iam_api_url(), "policy/user")),
            params={"user_nrn": user_nrn},
        )

    r = _request(ctx)
    process_response(r)



import os
import typing

import httpx
import typer

from neosctl import util
from neosctl.auth import ensure_login
from neosctl.util import process_response

app = typer.Typer()
secret_app = typer.Typer()
app.add_typer(secret_app, name="secret", help="Manage secrets for a spark job.")


def spark_url(name: str, gateway_api_url: str) -> str:
    return "{}/spark/{}".format(gateway_api_url.rstrip("/"), name)


def secret_url(name: str, gateway_api_url: str) -> str:
    return "{}/secret/{}".format(gateway_api_url.rstrip("/"), name)


@app.command()
def add_job(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    job_filepath: str = typer.Option(..., "--job-filepath", "-f"),
):
    """Assign a spark job.

    Assign and configure a spark job for a data product. This will result in a
    one off run of the spark job.
    """

    @ensure_login
    def _request(ctx: typer.Context, f: typing.IO) -> httpx.Response:
        return util.post(
            ctx,
            spark_url(product_name, ctx.obj.gateway_api_url),
            files={"spark_file": f},
        )

    fp = util.get_file_location(job_filepath)

    with fp.open("rb") as f:
        r = _request(ctx, f)

    process_response(r)


@app.command()
def generate_job(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    json_filepath: str = typer.Option(..., "--json-filepath", "-f"),
    dry_run: bool = typer.Option(
        False,
        "--dry-run",
        help="Generate spark code, but don't save the output.",
    ),
    reassign: bool = typer.Option(
        False,
        "--reassign",
        help="Generate spark code and replace existing spark job.",
    ),
):
    """Generate and assign a spark job.

    Generate a spark job from source/transformations configuration for a data product. This will result in a
    one off run of the spark job but no output.
    """

    @ensure_login
    def _request(ctx: typer.Context, data: typing.Dict) -> httpx.Response:
        return util.post(
            ctx,
            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/generate",
            json=data,
        )

    fp = util.get_file_location(json_filepath)
    data = util.load_json_file(fp, "builder")
    data["preview"] = dry_run
    data["reassign"] = reassign

    r = _request(ctx, data)

    process_response(r)


@app.command()
def fetch_job_preview(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Fetch output of generate dry-run."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/preview",
        )

    r = _request(ctx)

    process_response(r)


@app.command()
def job_history(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Get history of spark applications."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/history",
        )

    r = _request(ctx)

    process_response(r)


@app.command()
def run_history(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    suffix: str = typer.Option(None, "--suffix", "-s"),
):
    """Get history of spark application runs."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            "{spark_url}/history/{suffix}/run".format(
                spark_url=spark_url(product_name, ctx.obj.gateway_api_url),
                suffix=suffix,
            ),
        )

    r = _request(ctx)

    process_response(r)


@app.command()
def job_status(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    suffix: str = typer.Option(None, "--suffix", "-s"),
    run: str = typer.Option(None, "--run", "-r"),
):
    """Get status of a spark job.

    Defaults to current application status, previous application status or a
    specific scheduled run can be requested.
    """
    params = {
        k: v
        for k, v in {
            "suffix": suffix,
            "run": run,
        }.items()
        if v is not None
    }

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            f"{spark_url(product_name, ctx.obj.gateway_api_url)}",
            params=params,
        )

    r = _request(ctx)

    process_response(r)


def render_logs(payload: typing.Dict):
    return "\n".join(payload["logs"])


@app.command()
def job_logs(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    suffix: str = typer.Option(None, "--suffix", "-s"),
    run: str = typer.Option(None, "--run", "-r"),
):
    """Get logs for a spark job.

    Defaults to current application logs, previous application logs or a
    specific scheduled run can be requested.
    """
    params = {
        k: v
        for k, v in {
            "suffix": suffix,
            "run": run,
        }.items()
        if v is not None
    }

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/log",
            params=params,
        )

    r = _request(ctx)

    process_response(r, render_logs)


@app.command()
def update_job(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    job_filepath: str = typer.Option(None, "--job-filepath", "-f"),
):
    """Update an assigned spark job.

    Update the assigned spark job file and/or the spark job configuration values.
    """

    @ensure_login
    def _request(
        ctx: typer.Context,
        f: typing.Optional[typing.IO],
    ) -> httpx.Response:
        files = {"spark_file": f} if f else {}

        return util.put(
            ctx,
            spark_url(product_name, ctx.obj.gateway_api_url),
            files=files,
        )

    if job_filepath:
        fp = util.get_file_location(job_filepath)

        with fp.open("rb") as f:
            r = _request(ctx, f)
    else:
        r = _request(ctx, None)

    process_response(r)


@app.command()
def remove_job(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    force: bool = typer.Option(
        False,
        "--force",
        help="Force remove even if application is still running.",
    ),
):
    """Remove assigned spark job."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            spark_url(product_name, ctx.obj.gateway_api_url),
            params={"force": force},
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def trigger_job(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Trigger assigned spark job.

    Trigger an additional run of a spark job.
    """

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.post(
            ctx,
            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/trigger",
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def csv_job(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    delimiter: str = typer.Option(",", "--delimiter", "-d", help="csv delimiter"),
    quotechar: typing.Optional[str] = typer.Option(None, "--quote-char", "-q", help="csv quote char"),
    csv_write_mode: typing.Optional[str] = typer.Option(
        None,
        "--write-mode",
        "-w",
        help="csv write mode [overwrite|append]",
    ),
    job_filepath: str = typer.Option(..., "--job-filepath", "-f"),
):
    """Ingest a csv file into a data product.

    Upload a csv file for processing into a data product.
    """

    @ensure_login
    def _request(ctx: typer.Context, f: typing.IO) -> httpx.Response:
        params = {
            k: v
            for k, v in [
                ("delimiter", delimiter),
                ("quotechar", quotechar),
                ("csv_write_mode", csv_write_mode),
            ]
            if v is not None
        }

        return util.post(
            ctx,
            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/csv",
            params=params,
            files={"csv_file": f},
        )

    fp = util.get_file_location(job_filepath)

    with fp.open("rb") as f:
        r = _request(ctx, f)

    process_response(r)


@app.command()
def schedule_job(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    schedule: str = typer.Option(..., "--schedule", "-s", help='Schedule in crontab format (e.g. "* * * * *")'),
):
    """Schedule an assigned spark job.

    Schedule a spark job once it is configured correctly to run periodically.
    """

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.post(
            ctx,
            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/scheduled",
            json={
                "cron_expression": schedule,
            },
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def reschedule_job(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    schedule: str = typer.Option(..., "--schedule", "-s", help='Schedule in crontab format (e.g. "* * * * *")'),
):
    """Reschedule an assigned spark job.

    Update existing scheduled spark job run schedule.
    """

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.put(
            ctx,
            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/scheduled",
            json={
                "cron_expression": schedule,
            },
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def unschedule_job(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Unschedule a scheduled spark job.

    Remove existing scheduled spark job.
    """

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/scheduled",
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def stream_job(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    trigger_interval: str = typer.Option(
        ...,
        "--trigger",
        "-t",
        help='Trigger interval in spark format (e.g. "30 seconds")',
    ),
):
    """Run an infinite spark job.

    For streaming data product spark job will be created with given trigger interval.
    """

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        params = {"trigger_interval": trigger_interval}

        return util.post(
            ctx,
            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/streaming",
            params=params,
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def restream_job(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    trigger_interval: str = typer.Option(
        ...,
        "--trigger",
        "-t",
        help='Trigger interval in spark format (e.g. "30 seconds")',
    ),
):
    """Update an infinite spark job.

    For streaming data product spark job will be removed and recreated with given trigger interval.
    """

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        params = {"trigger_interval": trigger_interval}

        return util.put(
            ctx,
            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/streaming",
            params=params,
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def unstream_job(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Pause an infinite spark job.

    For streaming data product spark job will be removed and recreated with trigger interval "once".
    """

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:

        return util.delete(
            ctx,
            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/streaming",
        )

    r = _request(ctx)
    process_response(r)


@secret_app.command()
def add(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    secrets: typing.List[str] = typer.Option(..., "--secret", "-s", help="Secret in the form key:value"),
):
    """Add a set of secrets for a spark job."""

    @ensure_login
    def _request(ctx: typer.Context, payload: typing.Dict) -> httpx.Response:
        return util.post(
            ctx,
            secret_url(product_name, ctx.obj.gateway_api_url),
            json=payload,
        )

    payload = {"data": {}}
    for s in secrets:
        name, value = s.split(":")
        payload["data"][name] = value

    r = _request(ctx, payload)

    process_response(r)


@secret_app.command()
def update(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    secrets: typing.List[str] = typer.Option(..., "--secret", "-s", help="Secret in the form key:value"),
):
    """Update existing secrets.

    This will overwrite existing keys, and add new keys, any keys that already
    exist but aren't provided will remain.
    """

    @ensure_login
    def _request(ctx: typer.Context, payload: typing.Dict) -> httpx.Response:
        return util.patch(
            ctx,
            secret_url(product_name, ctx.obj.gateway_api_url),
            json=payload,
        )

    payload = {"data": {}}
    for s in secrets:
        name, value = s.split(":")
        payload["data"][name] = value

    r = _request(ctx, payload)

    process_response(r)


@secret_app.command()
def remove(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Remove secret."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            secret_url(product_name, ctx.obj.gateway_api_url),
        )

    r = _request(ctx)

    process_response(r)


@secret_app.command()
def remove_key(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    keys: typing.List[str] = typer.Option(..., "--key", "-k", help="Key name you wish to remove from secret"),
):
    """Remove a set of keys from a secret."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            f"{secret_url(product_name, ctx.obj.gateway_api_url)}/key",
            json={"keys": keys},
        )

    r = _request(ctx)

    process_response(r)


@secret_app.command()
def get(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Get existing secret keys."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            secret_url(product_name, ctx.obj.gateway_api_url),
        )

    r = _request(ctx)

    process_response(r)



import json
import os
import pathlib
import typing

import httpx
import typer

from neosctl import constant, util
from neosctl.auth import ensure_login
from neosctl.schema import DataProductCreate
from neosctl.util import process_response

app = typer.Typer()


def product_url(ctx: typer.Context) -> str:
    return "{}/product".format(ctx.obj.get_gateway_api_url().rstrip("/"))


special_delimiters = {
    r"\t": "\t",
}


@app.command(name="template")
def template(
    ctx: typer.Context,
    name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    filepath: str = typer.Option(..., "--filepath", "-f", help="Filepath of the csv template"),
    output_dir: str = typer.Option(..., "--output-dir", "-o", help="Output directory for the json template"),
    delimiter: str = typer.Option(",", "--delimiter", "-d", help="csv delimiter"),
    quotechar: typing.Optional[str] = typer.Option(None, "--quote-char", "-q", help="csv quote char"),
):
    """Generate a data product schema template from a csv.

    Given a csv with a header row, generate a template field schema.
    """

    @ensure_login
    def _request(ctx: typer.Context, f: typing.IO) -> httpx.Response:
        params = {k: v for k, v in [("delimiter", delimiter), ("quotechar", quotechar)] if v is not None}

        return util.post(
            ctx,
            f"{product_url(ctx)}/template",
            params=params,
            files={"csv_file": f},
        )

    fp = util.get_file_location(filepath)

    with fp.open("rb") as f:
        r = _request(ctx, f)

    if r.status_code >= constant.BAD_REQUEST_CODE:
        process_response(r)

    fp = pathlib.Path(output_dir) / f"{name}.json"
    with fp.open("w") as f:
        json.dump(r.json(), f, indent=4)


@app.command(name="create")
def create_from_json(
    ctx: typer.Context,
    name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    engine: str = typer.Option(..., "--engine", "-e", help="Storage engine"),
    filepath: str = typer.Option(..., "--filepath", "-f", help="Filepath of the table schema json payload"),
):
    """Create a data product."""

    @ensure_login
    def _request(ctx: typer.Context, dpc: DataProductCreate) -> httpx.Response:
        return util.post(
            ctx,
            f"{product_url(ctx)}/{name}",
            json=dpc.dict(exclude_none=True, by_alias=True),
        )

    fp = util.get_file_location(filepath)
    fields = util.load_json_file(fp, "schema")

    dpc = DataProductCreate(engine=engine, fields=fields)

    r = _request(ctx, dpc)
    process_response(r)


@app.command(name="list")
def list_products(ctx: typer.Context):
    """List data products."""

    @ensure_login
    def _request(ctx: typer.Context):
        return util.get(ctx, product_url(ctx))

    r = _request(ctx)
    process_response(r)


@app.command()
def delete_data(
    ctx: typer.Context,
    name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Delete data from a data product."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            f"{product_url(ctx)}/{name}/data",
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def delete(
    ctx: typer.Context,
    name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    force: bool = typer.Option(
        False,
        "--force",
        help="Force remove even if attached spark application is still running.",
    ),
):
    """Delete a data product."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            f"{product_url(ctx)}/{name}",
            params={"force": force},
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def publish(
    ctx: typer.Context,
    name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Publish a data product."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.post(
            ctx,
            f"{product_url(ctx)}/{name}/publish",
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def unpublish(
    ctx: typer.Context,
    name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Unpublish a product."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            f"{product_url(ctx)}/{name}/publish",
        )

    r = _request(ctx)
    process_response(r)


@app.command(name="get")
def get_product(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Get data product schema."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            f"{product_url(ctx)}/{product_name}",
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def preview(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Preview data product data.

    Get the first 25 rows of a data product's data.
    """

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            "{product_url}/{name}/data".format(
                product_url=product_url(ctx),
                name=product_name,
            ),
        )

    r = _request(ctx)
    process_response(r)



import httpx
import typer

from neosctl import util
from neosctl.auth import ensure_login
from neosctl.schema import RegisterCore, RemoveCore
from neosctl.util import process_response

app = typer.Typer()


def core_url(registry_api_url: str) -> str:
    return "{}/core".format(registry_api_url.rstrip("/"))


def data_product_url(registry_api_url: str, postfix: str = "") -> str:
    return "{}/data_product{}".format(registry_api_url.rstrip("/"), postfix)


@app.command(name="register-core")
def register_core(
    ctx: typer.Context,
    partition: str = typer.Argument(..., help="Core partition"),
    name: str = typer.Argument(..., help="Core name"),
):
    """Register a core.

    Register a core to receive an identifier and access key for use in deployment.
    """

    @ensure_login
    def _request(ctx: typer.Context, rc: RegisterCore) -> httpx.Response:
        return util.post(
            ctx,
            core_url(ctx.obj.get_registry_api_url()),
            json=rc.dict(exclude_none=True),
        )

    rc = RegisterCore(partition=partition, name=name)

    r = _request(ctx, rc)
    process_response(r)


@app.command(name="list-cores")
def list_cores(
    ctx: typer.Context,
):
    """List existing registered cores."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            core_url(ctx.obj.get_registry_api_url()),
        )

    r = _request(ctx)
    process_response(r)


@app.command(name="remove-core")
def remove_core(
    ctx: typer.Context,
    urn: str = typer.Argument(..., help="Core urn"),
):
    """Remove a registered core."""

    @ensure_login
    def _request(ctx: typer.Context, rc: RemoveCore) -> httpx.Response:
        return util.delete(
            ctx,
            url=core_url(ctx.obj.get_registry_api_url()),
            json=rc.dict(exclude_none=True),
        )

    rc = RemoveCore(urn=urn)

    r = _request(ctx, rc)
    process_response(r)


@app.command(name="search")
def search_products(
    ctx: typer.Context,
    search_term: str,
):
    """Search published data products across cores."""

    @ensure_login
    def _request(ctx: typer.Context, search_term: str) -> httpx.Response:
        return util.get(
            ctx,
            data_product_url(ctx.obj.get_registry_api_url(), "/search"),
            params={"search_term": search_term},
        )

    r = _request(ctx, search_term)
    process_response(r)


@app.command(name="get-product")
def get_product(
    ctx: typer.Context,
    urn: str,
):
    """Get data product details."""

    @ensure_login
    def _request(ctx: typer.Context, urn: str) -> httpx.Response:
        return util.get(
            ctx,
            data_product_url(ctx.obj.get_registry_api_url(), f"/urn/{urn}"),
        )

    r = _request(ctx, urn)
    process_response(r)



import re
import typing

import click
import typer

from neosctl import schema
from neosctl.util import get_user_profile_section, prettify_json, remove_config, send_output, upsert_config

app = typer.Typer()


r = re.compile(r"http[s]?:\/\/.*")


def validate_url(value):
    m = r.match(value)
    if m is None:
        msg = f"Invalid url, must match pattern: `{r}`."
        raise click.UsageError(msg)
    return value


@app.command()
def init(
    ctx: typer.Context,
    hostname: typing.Optional[str] = typer.Option(None, "--host", "-h"),
    gateway_api_url: typing.Optional[str] = typer.Option(None, "--gateway-api-url", "-g"),
    registry_api_url: typing.Optional[str] = typer.Option(None, "--registry-api-url", "-r"),
    iam_api_url: typing.Optional[str] = typer.Option(None, "--iam-api-url", "-i"),
    storage_api_url: typing.Optional[str] = typer.Option(None, "--storage-api-url", "-s"),
    username: typing.Optional[str] = typer.Option(None, "--username", "-u"),
    ignore_tls: bool = typer.Option(
        False,
        "--ignore-tls",
        help="Ignore TLS errors (useful in local/development environments",
    ),
    non_interactive: bool = typer.Option(
        False,
        "--non-interactive",
        help="Don't ask for input, generate api values based on hostname.",
    ),
):
    """Initialise a profile.

    Create a profile that can be reused in later commands to define which
    services to interact with, and which user to interact as.

    Call `init` on an existing profile will update the existing profile.
    """
    typer.echo(f"Initialising [{ctx.obj.profile_name}] profile.")
    default_host = f"{hostname}/api/{{}}" if hostname else ""

    if non_interactive and not (hostname and username):
        send_output(
            msg="\nError: --hostname/-h and --username/-u required for non-interactive mode.",
            exit_code=1,
        )

    if non_interactive:
        gateway_api_url = gateway_api_url or f"{hostname}/api/gateway"
        storage_api_url = storage_api_url or f"{hostname}/api/storage"
        iam_api_url = iam_api_url or f"{hostname}/api/iam"
        registry_api_url = registry_api_url or f"{hostname}/api/registry"

    urls = {
        "gateway_api_url": gateway_api_url,
        "registry_api_url": registry_api_url,
        "iam_api_url": iam_api_url,
        "storage_api_url": storage_api_url,
    }
    for key, default, prompt in [
        ("gateway", ctx.obj.get_gateway_api_url(), "Gateway API url"),
        ("registry", ctx.obj.get_registry_api_url(), "Registry API url"),
        ("iam", ctx.obj.get_iam_api_url(), "IAM API url"),
        ("storage", ctx.obj.get_storage_api_url(), "Storage API url"),
    ]:
        url_key = f"{key}_api_url"
        if urls[url_key] is None:
            urls[url_key] = typer.prompt(
                prompt,
                default=default or default_host.format(key),
                value_proc=validate_url,
            )
        else:
            validate_url(urls[url_key])

    if username is None:
        kwargs = {}
        if ctx.obj.profile:
            kwargs["default"] = ctx.obj.profile.user
        username = typer.prompt(
            "Username",
            **kwargs,
        )

    profile = schema.Profile(  # nosec: B106
        user=username,
        access_token="",
        refresh_token="",
        ignore_tls=ignore_tls,
        **urls,
    )

    upsert_config(ctx, profile)


@app.command()
def delete(
    ctx: typer.Context,
):
    """Delete a profile."""
    typer.confirm(f"Remove [{ctx.obj.profile_name}] profile", abort=True)
    remove_config(ctx)


@app.command()
def view(
    ctx: typer.Context,
):
    """View configuration for a profile."""
    send_output(
        msg=prettify_json({**get_user_profile_section(ctx.obj.config, ctx.obj.profile_name)}),
    )


@app.command(name="list")
def list_profiles(
    ctx: typer.Context,
):
    """List available profiles."""
    send_output(
        msg=prettify_json(sorted(ctx.obj.config.sections())),
    )



import typing

import click
import typer

from neosctl import schema, util
from neosctl.util import (
    check_profile_exists,
    constant,
    get_user_profile,
    is_success_response,
    process_response,
    read_config_dotfile,
    send_output,
    upsert_config,
)

app = typer.Typer()


def auth_url(iam_api_url: str) -> str:
    return "{}".format(iam_api_url.rstrip("/"))


def _check_refresh_token_exists(ctx: typer.Context):
    if ctx.obj.profile.refresh_token == "":  # nosec: B105
        send_output(
            msg=f"You need to login. Run neosctl -p {ctx.obj.profile_name} auth login",
            exit_code=1,
        )

    return True


def ensure_login(method):
    def check_access_token(*args, **kwargs):
        ctx = args[0]
        if not isinstance(ctx, click.core.Context):
            # Developer reminder
            msg = "First argument should be typer.Context instance"
            raise TypeError(msg)

        r = method(*args, **kwargs)

        check_profile_exists(ctx)

        # Try to refresh token
        # Confirm it is a token invalid 401, registry not configured mistriggers this flow.
        if r.status_code == constant.UNAUTHORISED_CODE:
            data = r.json()
            if "code" in data and data["code"].startswith("A0"):
                refresh_token(ctx)

                # Refresh the context
                c = read_config_dotfile()
                ctx.obj.config = c
                ctx.obj.profile = get_user_profile(c, ctx.obj.profile_name)

                r = method(*args, **kwargs)

        return r  # noqa: RET504

    return check_access_token


def _update_profile(
    ctx: typer.Context,
    auth: schema.Auth = schema.Auth(),
):
    return schema.Profile(
        gateway_api_url=ctx.obj.gateway_api_url,
        registry_api_url=ctx.obj.registry_api_url,
        iam_api_url=ctx.obj.iam_api_url,
        storage_api_url=ctx.obj.storage_api_url,
        user=ctx.obj.profile.user,
        access_token=auth.access_token,
        refresh_token=auth.refresh_token,
        ignore_tls=ctx.obj.profile.ignore_tls,
    )


@app.command()
def login(
    ctx: typer.Context,
    password: typing.Optional[str] = typer.Option(None, "--password", "-p"),
):
    """Login to neos."""
    check_profile_exists(ctx)

    if password is None:
        password = typer.prompt(
            "[{profile}] Enter password for user ({user})".format(
                profile=ctx.obj.profile_name,
                user=ctx.obj.profile.user,
            ),
            hide_input=True,
        )

    r = util.post(
        ctx,
        f"{auth_url(ctx.obj.get_iam_api_url())}/login",
        json={"user": ctx.obj.profile.user, "password": password},
    )

    if not is_success_response(r):
        process_response(r)

    upsert_config(ctx, _update_profile(ctx, schema.Auth(**r.json())))

    send_output(
        msg="Login success",
        exit_code=0,
    )


@app.command()
def logout(ctx: typer.Context):
    """Logout from neos."""
    check_profile_exists(ctx)

    _check_refresh_token_exists(ctx)

    r = util.post(
        ctx,
        f"{auth_url(ctx.obj.get_iam_api_url())}/logout",
        json={"refresh_token": ctx.obj.profile.refresh_token},
    )

    if not is_success_response(r):
        process_response(r)

    upsert_config(ctx, _update_profile(ctx, schema.Auth()))

    send_output(
        msg="Logout success",
        exit_code=0,
    )


def refresh_token(ctx: typer.Context):
    check_profile_exists(ctx)
    _check_refresh_token_exists(ctx)

    r = util.post(
        ctx,
        f"{auth_url(ctx.obj.get_iam_api_url())}/refresh",
        json={"refresh_token": ctx.obj.profile.refresh_token},
    )

    if not is_success_response(r):
        process_response(r)

    upsert_config(ctx, _update_profile(ctx, schema.Auth(**r.json())))

    return r



import os

import httpx
import typer

from neosctl import util
from neosctl.auth import ensure_login
from neosctl.util import process_response

app = typer.Typer()

product_app = typer.Typer()
tag_app = typer.Typer()
pipeline_app = typer.Typer()
browse_app = typer.Typer()

app.add_typer(product_app, name="product", help="Manage product metadata.")
app.add_typer(tag_app, name="tag", help="Manage tags.")
app.add_typer(browse_app, name="browse", help="Browse external data source metadata.")
app.add_typer(pipeline_app, name="pipeline", help="Manage data source pipelines.")


def metadata_url(ctx: typer.Context, postfix: str = "") -> str:
    return "{}/metadata{}".format(ctx.obj.get_gateway_api_url().rstrip("/"), postfix)


@product_app.command()
def get(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Get data product metadata."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            metadata_url(ctx, f"/product/{product_name}"),
        )

    r = _request(ctx)

    process_response(r)


@tag_app.command("add")
def add_tag(
    ctx: typer.Context,
    tag: str,
    scope: str = typer.Option("field", "--scope", "-s"),
):
    """Add a tag."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.post(
            ctx,
            metadata_url(ctx, "/tag"),
            json={
                "tag": tag.lower(),
                "scope": scope.upper(),
            },
        )

    r = _request(ctx)

    process_response(r)


@tag_app.command("remove")
def remove_tag(
    ctx: typer.Context,
    tag: str,
    scope: str = typer.Option("field", "--scope", "-s"),
):
    """Remove a tag."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            url=metadata_url(ctx, "/tag"),
            json={
                "tag": tag.lower(),
                "scope": scope.upper(),
            },
        )

    r = _request(ctx)

    process_response(r)


@tag_app.command("list")
def list_tags(
    ctx: typer.Context,
    tag_filter: str = typer.Option(None, "--tag-filter", "-t"),
    scope: str = typer.Option("field", "--scope", "-s"),
    system_defined: bool = typer.Option(False, "--system-defined"),
):
    """List tags.

    Tag list can be filtered using `-t/--tag-filter`, and the tag scope can be
    defined using `-s/--scope [PRODUCT|FIELD]`.
    """

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        params = {"scope": scope.upper(), "system_defined": system_defined}
        if tag_filter:
            params["tag_filter"] = tag_filter
        return util.get(
            ctx,
            url=metadata_url(ctx, "/tag"),
            params=params,
        )

    r = _request(ctx)

    process_response(r)


@browse_app.command()
def source_types(ctx: typer.Context):
    """List source types."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            url=metadata_url(ctx, "/browse"),
        )

    r = _request(ctx)

    process_response(r)


@browse_app.command("databases")
def browse_databases(
    ctx: typer.Context,
    source_type: str = typer.Option(..., "--source-type", "-s"),
):
    """List databases by source type."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            url=metadata_url(ctx, f"/browse/{source_type}"),
        )

    r = _request(ctx)

    process_response(r)


@browse_app.command("datasets")
def browse_datasets(
    ctx: typer.Context,
    source_type: str = typer.Option(..., "--source-type", "-s"),
    database: str = typer.Option(..., "--database", "-d"),
):
    """List datasets by source type and database."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            url=metadata_url(ctx, f"/browse/{source_type}/{database}"),
        )

    r = _request(ctx)

    process_response(r)


@browse_app.command("dataset-metadata")
def browse_dataset_metadata(
    ctx: typer.Context,
    source_type: str = typer.Option(..., "--source-type", "-s"),
    database: str = typer.Option(..., "--database", "-d"),
    dataset_urn: str = typer.Option(..., "--dataset-urn", "-du"),
):
    """Get dataset metadata by source type, database and dataset urn."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            url=metadata_url(ctx, f"/browse/{source_type}/{database}/{dataset_urn}"),
        )

    r = _request(ctx)

    process_response(r)


@pipeline_app.command("add")
def add_pipeline(
    ctx: typer.Context,
    name: str = typer.Option(..., "--name", "-n"),
    schedule: str = typer.Option(
        ...,
        "--schedule",
        "-s",
        help='Pipeline schedule in crontab format (e.g. "* * * * *")',
    ),
    schedule_timezone: str = typer.Option(..., "--schedule-timezone", "-st"),
    source: str = typer.Option(
        ...,
        "--source",
        "-sr",
        help="Data source DSN (e.g. mysql://user:pass@host:3306/dbname)",
    ),
    include_tables: bool = typer.Option(True, "--include-tables", "-it", help="Applies only to relational DBs"),
    include_views: bool = typer.Option(True, "--include-views", "-iv", help="Applies only to relational DBs"),
):
    """Add an ingestion pipeline."""
    source_url = httpx.URL(source)
    request_data = {
        "pipeline_type": source_url.scheme,
        "pipeline_name": name,
        "schedule": schedule,
        "schedule_timezone": schedule_timezone,
        "source": {
            "host": source_url.netloc.decode(),
            "database": source_url.path.lstrip("/"),
            "username": source_url.username,
            "password": source_url.password,
            "include_tables": include_tables,
            "include_views": include_views,
        },
    }

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.post(
            ctx,
            metadata_url(ctx, "/pipeline"),
            json=request_data,
        )

    r = _request(ctx)

    process_response(r)


@pipeline_app.command("remove")
def remove_pipeline(
    ctx: typer.Context,
    name: str = typer.Option(..., "--name", "-n"),
):
    """Remove an ingestion pipeline."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            metadata_url(ctx, "/pipeline"),
            json={
                "pipeline_name": name,
            },
        )

    r = _request(ctx)

    process_response(r)


@product_app.command("description")
def product_description(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    description: str = typer.Option(..., "--description", "-d"),
):
    """Add a product description."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.post(
            ctx,
            metadata_url(ctx, f"/product/{product_name}/description"),
            json={
                "description": description,
            },
        )

    r = _request(ctx)

    process_response(r)


@product_app.command("tag")
def product_tag(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    tag: str = typer.Option(..., "--tag", "-t"),
):
    """Add a product tag."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.post(
            ctx,
            metadata_url(ctx, f"/product/{product_name}/tag"),
            json={
                "tag": tag.lower(),
            },
        )

    r = _request(ctx)

    process_response(r)


@product_app.command("remove-tag")
def remove_product_tag(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    tag: str = typer.Option(..., "--tag", "-t"),
):
    """Remove a product tag."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            url=metadata_url(ctx, f"/product/{product_name}/tag"),
            json={
                "tag": tag.lower(),
            },
        )

    r = _request(ctx)

    process_response(r)


@product_app.command("field-tag")
def product_field_tag(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    field: str = typer.Option(..., "--field", "-f"),
    tag: str = typer.Option(..., "--tag", "-t"),
):
    """Add a product field tag."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.post(
            ctx,
            metadata_url(ctx, f"/product/{product_name}/{field}/tag"),
            json={
                "tag": tag.lower(),
            },
        )

    r = _request(ctx)

    process_response(r)


@product_app.command("remove-field-tag")
def remove_product_field_tag(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    field: str = typer.Option(..., "--field", "-f"),
    tag: str = typer.Option(..., "--tag", "-t"),
):
    """Remove a product field tag."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            url=metadata_url(ctx, f"/product/{product_name}/{field}/tag"),
            json={
                "tag": tag.lower(),
            },
        )

    r = _request(ctx)

    process_response(r)


@product_app.command("field-description")
def product_field_description(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
    field: str = typer.Option(..., "--field", "-f"),
    description: str = typer.Option(..., "--description", "-d"),
):
    """Add a product field description."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.post(
            ctx,
            metadata_url(ctx, f"/product/{product_name}/{field}/description"),
            json={
                "description": description,
            },
        )

    r = _request(ctx)

    process_response(r)


@product_app.command("quality-expectations")
def product_quality_expectations(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Get data product quality expectations."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            metadata_url(ctx, f"/product/{product_name}/quality/expectations"),
        )

    r = _request(ctx)

    process_response(r)


@product_app.command("quality-profiling")
def product_quality_profiling(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Get data product quality profiling."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            metadata_url(ctx, f"/product/{product_name}/quality/profiling"),
        )

    r = _request(ctx)

    process_response(r)


@product_app.command("quality-validations")
def product_quality_validations(
    ctx: typer.Context,
    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
):
    """Get data product quality validations."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            metadata_url(ctx, f"/product/{product_name}/quality/validations"),
        )

    r = _request(ctx)

    process_response(r)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['neosctl']

package_data = \
{'': ['*']}

install_requires = \
['Pygments>=2.13.0,<3.0.0',
 'httpx>=0.23.0,<0.24.0',
 'pydantic>=1.9.2,<2.0.0',
 'typer>=0.6.1,<0.7.0']

entry_points = \
{'console_scripts': ['neosctl = neosctl.cli:app']}

setup_kwargs = {
    'name': 'neosctl',
    'version': '0.4.5',
    'description': 'Nortal Core CLI',
    'long_description': '# Core CLI v0.4.5\n\n## Prerequisites\n\nThe following packages are used across python repositories. A global install of them all is _highly_ recommended.\n\n- [Poetry](https://python-poetry.org/docs/#installation)\n- [Invoke](https://www.pyinvoke.org/installing.html)\n- [Kubefwd](https://kubefwd.com)\n\nA running cluster from [Local\nHelm](https://github.com/NEOM-KSA/neos-core-platform/tree/main/demo/helm) with\n`gateway` service port forwarded. Details on port forwarding below.\n\n### WSL\n\nIf running on Windows, you may need to install `distutils` to install the service.\n\n```bash\n$ sudo apt-get install python3.10-distutils\n```\n\n## Initial setup\n\n```bash\n$ invoke install-dev\n```\n\n## Code Quality\n\n### Tests\n\n```bash\ninvoke tests\ninvoke tests-coverage\n```\n\n## Linting\n\n```bash\ninvoke check-style\ninvoke isort\n```\n\n## Running locally\n\n### Port forwarding\n\nTo access the gateway api locally, you will need to connect to the pod inside\nthe cluster using `kubefwd`.\n\n```bash\n$ sudo kubefwd svc -n core -c ~/.kube/config\n```\n\n## Neosctl\n\nWhen running locally, if you do not manage your own virtual environments, you\ncan use poetry to put you in a shell with access to the installed code.\n\n```bash\n$ poetry shell\n```\n\n### Initialize profile\n\n```bash\n$ neosctl -p my-profile profile init\nInitialising [default] profile.\nGateway API url [http://core-gateway.core-gateway:9000/api/gateway]: <http://gateway_api_url:port>\nRegistry API url [http://neos-registry.registry:80/api/registry]: <http://registry_api_url:port>\nIAM API url [http://core-iam.core-iam:80/api/iam]: <http://iam_api_url:port>\nStorage API url [http://core-storage.core-storage:9000/api/storage]: <http://storage_api_url:port>\nUsername: <username>\n```\n\n```bash\n$ cat ~/.neosctl\n```\n\nTo work with the same profile across multiple commands you can export the\nprofile name as an environment variable.\n\n```bash\n$ neosctl -p my-profile product list\n...\n$ export NEOSCTL_PROFILE=my-profile\n$ neosctl product list\n```\n\n### Login\n\n```bash\n$ neosctl -p=<my-profile> auth login\n```\n\n### Commands to work with data products\n\n```bash\n$ neosctl --help\n$ neosctl product --help\n$ neosctl metadata --help\n```\n\nTo work with the same product across multiple commands you can export the\nproduct name as an environment variable.\n\n```bash\n$ neosctl product get my-data-product\n...\n$ export NEOSCTL_PRODUCT=my-data-product\n$ neosctl product get\n```\n\n## Releases\n\nRelease management is handled using `bump2version`. The below commands will tag\na new release. This will also update the helm chart version, this should not be\nmanually changed.\n\n```bash\n$ invoke bump-patch\n$ invoke bump-minor\n$ invoke bump-major\n> vX.Y.Z\n```\n',
    'author': 'None',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/NEOS-Critical/neos-platform-cli',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import configparser
import os
import typing

import typer

import neosctl
from neosctl import auth, constant, consume, iam, metadata, product, profile, registry, schema, spark, storage
from neosctl.util import get_user_profile, read_config_dotfile


def _generate_common_schema(
    gateway_api_url: str,
    registry_api_url: str,
    iam_api_url: str,
    storage_api_url: str,
    profile_name: str,
    config: configparser.ConfigParser,
    profile: schema.Profile = None,
):
    common_schema = schema.Common(
        gateway_api_url=gateway_api_url,
        registry_api_url=registry_api_url,
        iam_api_url=iam_api_url,
        storage_api_url=storage_api_url,
        profile_name=profile_name,
        config=config,
        profile=profile,
    )
    common_schema.gateway_api_url = common_schema.get_gateway_api_url()
    common_schema.registry_api_url = common_schema.get_registry_api_url()
    common_schema.iam_api_url = common_schema.get_iam_api_url()
    common_schema.storage_api_url = common_schema.get_storage_api_url()

    return common_schema


def version_callback(value: bool):
    if value:
        typer.echo(f"neosctl {neosctl.__version__}")
        raise typer.Exit()


def callback(
    ctx: typer.Context,
    version: typing.Optional[bool] = typer.Option(  # noqa: ARG001
        None,
        "--version",
        callback=version_callback,
        help="Print version and exit.",
    ),
    gateway_api_url: str = typer.Option("", "--gateway-api-url", "--gurl", help="Gateway API URL"),
    registry_api_url: str = typer.Option("", "--registry-api-url", "--rurl", help="Registry API URL"),
    iam_api_url: str = typer.Option("", "--iam-api-url", "--iurl", help="IAM API URL"),
    storage_api_url: str = typer.Option("", "--storage-api-url", "--surl", help="Storage API URL"),
    profile: str = typer.Option(
        os.getenv("NEOSCTL_PROFILE", constant.DEFAULT_PROFILE),
        "--profile",
        "-p",
        help="Profile name",
    ),
):
    config = read_config_dotfile()
    user_profile = get_user_profile(config, profile, allow_missing=True)

    common_schema = _generate_common_schema(
        gateway_api_url=gateway_api_url,
        registry_api_url=registry_api_url,
        iam_api_url=iam_api_url,
        storage_api_url=storage_api_url,
        profile_name=profile,
        config=config,
        profile=user_profile,
    )

    ctx.obj = common_schema


def common(
    ctx: typer.Context,
):
    user_profile = get_user_profile(ctx.obj.config, ctx.obj.profile_name)
    ctx.obj.profile = user_profile


app = typer.Typer(name="neosctl", callback=callback)
app.add_typer(profile.app, name="profile", help="Manage profiles.")
app.add_typer(auth.app, name="auth", callback=common, help="Manage authentication status.")
app.add_typer(consume.app, name="consume", callback=common, help="Consume published data products.")
app.add_typer(iam.app, name="iam", callback=common, help="Manage access policies.")
app.add_typer(storage.app, name="storage", callback=common, help="Interact with Storage (as a service).")
app.add_typer(metadata.app, name="metadata", callback=common, help="Manage and browse metadata.")
app.add_typer(product.app, name="product", callback=common, help="Manage data products.")
app.add_typer(registry.app, name="registry", callback=common, help="Manage cores and search data products.")
app.add_typer(spark.app, name="spark", callback=common, help="Manage data product spark/secrets.")
